@@ -49,9 +49,10 @@ ArvStream *	arv_camera_create_stream	(ArvCamera *camera, ArvStreamCallback callb
 
 /* Device informations */
 
-const char *	arv_camera_get_vendor_name	(ArvCamera *camera, GError **error);
-const char *	arv_camera_get_model_name	(ArvCamera *camera, GError **error);
-const char *	arv_camera_get_device_id	(ArvCamera *camera, GError **error);
+const char *	arv_camera_get_vendor_name		(ArvCamera *camera, GError **error);
+const char *	arv_camera_get_model_name		(ArvCamera *camera, GError **error);
+const char * 	arv_camera_get_device_serial_number 	(ArvCamera *camera, GError **error);
+const char *	arv_camera_get_device_id		(ArvCamera *camera, GError **error);
 
 /* Image format control */
 
@@ -166,6 +167,11 @@ const char ** 	arv_camera_dup_available_enumerations_as_display_names 	(ArvCamer
 
 gboolean 	arv_camera_is_feature_available 	(ArvCamera *camera, const char *feature, GError **error);
 
+/* Runtime policies */
+
+void		arv_camera_set_register_cache_policy	(ArvCamera *camera, ArvRegisterCachePolicy policy);
+void		arv_camera_set_range_check_policy	(ArvCamera *camera, ArvRangeCheckPolicy policy);
+
 /* GigEVision specific API */
 
 gboolean	arv_camera_is_gv_device			(ArvCamera *camera);
@@ -194,6 +200,8 @@ void            arv_camera_uv_get_bandwidth_bounds      	(ArvCamera *camera, gui
 
 /* Chunk data */
 
+gboolean		arv_camera_are_chunks_available		(ArvCamera *camera, GError **error);
+
 void 			arv_camera_set_chunk_mode 	(ArvCamera *camera, gboolean is_active, GError **error);
 gboolean 		arv_camera_get_chunk_mode 	(ArvCamera *camera, GError **error);
 void 			arv_camera_set_chunk_state 	(ArvCamera *camera, const char *chunk, gboolean is_enabled, GError **error);
