@@ -20,6 +20,7 @@
  * Author: Emmanuel Pacaud <emmanuel@gnome.org>
  */
 
+#include <arvdebugprivate.h>
 #include <arv.h>
 #include <stdlib.h>
 #include <string.h>
@@ -28,24 +29,42 @@
 static char *arv_option_device_name = NULL;
 static char *arv_option_device_address = NULL;
 static char *arv_option_debug_domains = NULL;
-static char *arv_option_cache_policy = NULL;
-static char *arv_option_range_check_policy = NULL;
+static char *arv_option_register_cache = NULL;
+static char *arv_option_range_check = NULL;
 static gboolean arv_option_show_time = FALSE;
 
 static const GOptionEntry arv_option_entries[] =
 {
-	{ "name",		'n', 0, G_OPTION_ARG_STRING,
-		&arv_option_device_name,	NULL, "<device_name>"},
-	{ "address",		'a', 0, G_OPTION_ARG_STRING,
-		&arv_option_device_address,	NULL, "<device_address>"},
-	{ "cache-policy",	'c', 0, G_OPTION_ARG_STRING,
-		&arv_option_cache_policy, 	"Register cache policy", "[disable|enable|debug]" },
-	{ "range-check-policy",	'r', 0, G_OPTION_ARG_STRING,
-		&arv_option_range_check_policy,	"Range check policy", "[disable|enable]" },
-	{ "time",		't', 0, G_OPTION_ARG_NONE,
-		&arv_option_show_time, 		"Show execution time", NULL},
-	{ "debug", 		'd', 0, G_OPTION_ARG_STRING,
-		&arv_option_debug_domains, 	NULL, "<category>[:<level>][,...]" },
+	{
+		"name",				'n', 0, G_OPTION_ARG_STRING,
+		&arv_option_device_name,	NULL,
+		"<device_name>"
+	},
+	{
+		"address",			'a', 0, G_OPTION_ARG_STRING,
+		&arv_option_device_address,	NULL,
+		"<device_address>"
+	},
+	{
+		"register-cache",		'\0', 0, G_OPTION_ARG_STRING,
+		&arv_option_register_cache, 	"Register cache policy",
+		"{disable|enable|debug}"
+	},
+	{
+		"range-check",			'\0', 0, G_OPTION_ARG_STRING,
+		&arv_option_range_check,	"Range check policy",
+		"{disable|enable}"
+	},
+	{
+		"time",				't', 0, G_OPTION_ARG_NONE,
+		&arv_option_show_time, 		"Show execution time",
+		NULL
+	},
+	{
+		"debug", 			'd', 0, G_OPTION_ARG_STRING,
+		&arv_option_debug_domains, 	NULL,
+		"{<category>[:<level>][,...]|help}"
+	},
 	{ NULL }
 };
 
@@ -68,7 +87,7 @@ description_content[] =
 "arv-tool-" ARAVIS_API_VERSION " control Width=128 Height=128 Gain R[0x10000]=0x10\n"
 "arv-tool-" ARAVIS_API_VERSION " features\n"
 "arv-tool-" ARAVIS_API_VERSION " description Width Height\n"
-"arv-tool-" ARAVIS_API_VERSION " -n Basler-210ab4 genicam\n";
+"arv-tool-" ARAVIS_API_VERSION " -n Basler-210ab4 genicam";
 
 typedef enum {
 	ARV_TOOL_LIST_MODE_FEATURES,
@@ -195,7 +214,7 @@ arv_tool_list_features (ArvGc *genicam, const char *feature, ArvToolListMode lis
 
 static void
 arv_tool_execute_command (int argc, char **argv, ArvDevice *device,
-			  ArvRegisterCachePolicy cache_policy,
+			  ArvRegisterCachePolicy register_cache_policy,
 			  ArvRangeCheckPolicy range_check_policy)
 {
 	ArvGc *genicam;
@@ -205,7 +224,7 @@ arv_tool_execute_command (int argc, char **argv, ArvDevice *device,
 	if (device == NULL || argc < 2)
 		return;
 
-	arv_device_set_register_cache_policy (device, cache_policy);
+	arv_device_set_register_cache_policy (device, register_cache_policy);
 	arv_device_set_range_check_policy (device, range_check_policy);
 
 	genicam = arv_device_get_genicam (device);
@@ -384,8 +403,8 @@ int
 main (int argc, char **argv)
 {
 	ArvDevice *device;
-	ArvRegisterCachePolicy cache_policy = ARV_REGISTER_CACHE_POLICY_DEFAULT;
-	ArvRangeCheckPolicy range_check_policy = ARV_RANGE_CHECK_POLICY_DEFAULT;
+	ArvRegisterCachePolicy register_cache_policy;
+	ArvRangeCheckPolicy range_check_policy;
 	const char *device_id;
 	GOptionContext *context;
 	GError *error = NULL;
@@ -406,29 +425,39 @@ main (int argc, char **argv)
 
 	g_option_context_free (context);
 
-	if (arv_option_cache_policy == NULL ||
-	    g_strcmp0 (arv_option_cache_policy, "disable") == 0)
-		cache_policy = ARV_REGISTER_CACHE_POLICY_DISABLE;
-	else if (g_strcmp0 (arv_option_cache_policy, "enable") == 0)
-		cache_policy = ARV_REGISTER_CACHE_POLICY_ENABLE;
-	else if (g_strcmp0 (arv_option_cache_policy, "debug") == 0)
-		cache_policy = ARV_REGISTER_CACHE_POLICY_DEBUG;
+	if (arv_option_register_cache == NULL)
+		register_cache_policy = ARV_REGISTER_CACHE_POLICY_DEFAULT;
+	else if (g_strcmp0 (arv_option_register_cache, "disable") == 0)
+		register_cache_policy = ARV_REGISTER_CACHE_POLICY_DISABLE;
+	else if (g_strcmp0 (arv_option_register_cache, "enable") == 0)
+		register_cache_policy = ARV_REGISTER_CACHE_POLICY_ENABLE;
+	else if (g_strcmp0 (arv_option_register_cache, "debug") == 0)
+		register_cache_policy = ARV_REGISTER_CACHE_POLICY_DEBUG;
 	else {
-		printf ("Invalid cache policy\n");
+		printf ("Invalid register cache policy\n");
 		return EXIT_FAILURE;
 	}
 
-	if (arv_option_range_check_policy == NULL ||
-	    g_strcmp0 (arv_option_range_check_policy, "disable") == 0)
+	if (arv_option_range_check == NULL)
+		range_check_policy = ARV_RANGE_CHECK_POLICY_DEFAULT;
+	else if (g_strcmp0 (arv_option_range_check, "disable") == 0)
 		range_check_policy = ARV_RANGE_CHECK_POLICY_DISABLE;
-	else if (g_strcmp0 (arv_option_range_check_policy, "enable") == 0)
+	else if (g_strcmp0 (arv_option_range_check, "enable") == 0)
 		range_check_policy = ARV_RANGE_CHECK_POLICY_ENABLE;
+	else if (g_strcmp0 (arv_option_range_check, "debug") == 0)
+		range_check_policy = ARV_RANGE_CHECK_POLICY_DEBUG;
 	else {
 		printf ("Invalid range check policy\n");
 		return EXIT_FAILURE;
 	}
 
-	arv_debug_enable (arv_option_debug_domains);
+	if (!arv_debug_enable (arv_option_debug_domains)) {
+		if (g_strcmp0 (arv_option_debug_domains, "help") != 0)
+			printf ("Invalid debug selection\n");
+		else
+			arv_debug_print_infos ();
+		return EXIT_FAILURE;
+	}
 
 	device_id = arv_option_device_address != NULL ? arv_option_device_address : arv_option_device_name;
 	if (device_id != NULL) {
@@ -440,7 +469,8 @@ main (int argc, char **argv)
 			if (argc < 2)
 				printf ("%s\n", device_id);
 			else
-				arv_tool_execute_command (argc, argv, device, cache_policy, range_check_policy);
+				arv_tool_execute_command (argc, argv, device,
+							  register_cache_policy, range_check_policy);
 			g_object_unref (device);
 		} else {
 			fprintf (stderr, "Device '%s' not found%s%s\n", device_id,
@@ -461,7 +491,8 @@ main (int argc, char **argv)
 
 				if (ARV_IS_DEVICE (device)) {
 					printf ("%s (%s)\n", device_id, arv_get_device_address (i));
-					arv_tool_execute_command (argc, argv, device, cache_policy, range_check_policy);
+					arv_tool_execute_command (argc, argv, device,
+								  register_cache_policy, range_check_policy);
 
 					g_object_unref (device);
 				} else {
