@@ -49,7 +49,7 @@ main (int argc, char **argv)
 			if (val > max) max = val;
 		}
 		wt /= j;
-		printf ("SleepMeas: %6" G_GINT64_FORMAT " - Mean %7g Max %5g Min %5g rms %g\n",
+		g_print ("SleepMeas: %6" G_GINT64_FORMAT " - Mean %7g Max %5g Min %5g rms %g\n",
 			i, wt, max, min, sqrt(swt/j - wt*wt));
 	}
 
@@ -63,7 +63,7 @@ main (int argc, char **argv)
 			if( val > max ) max = val;
 		}
 		wt /= j;
-		printf ("TimeWait:  %6" G_GINT64_FORMAT " - Mean %7g Max %5g Min %5g rms %g\n",
+		g_print ("TimeWait:  %6" G_GINT64_FORMAT " - Mean %7g Max %5g Min %5g rms %g\n",
 			i, wt, max, min, sqrt(swt/j - wt*wt));
 	}
 
